﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Text;
using System.Windows.Forms;
using Newtonsoft.Json;
using RetailForce.Common.Validation;
using RetailForce.Fiscalisation;
using RetailForce.Fiscalisation.Configuration;
using RetailForce.Fiscalisation.Implementation.Germany;
using RetailForce.Fiscalisation.Model;
using RetailForce.Fiscalisation.Model.Document;
using RetailForce.Fiscalisation.ServiceClient;

namespace FiscalServiceClientSample.net2
{
    public partial class SampleForm : Form
    {
        private const string _fiscalClientUrl = "http://localhost:7678/api/v1/";
        private FiscalServiceClient _serviceClient = new FiscalServiceClient(_fiscalClientUrl);
        private FiscalClient _fiscalClient;
        private FiscalResponse _fiscalResponse;

        public SampleForm()
        {
            InitializeComponent();
        }

        private void SampleForm_Load(object sender, EventArgs e)
        {
            cmdReload_Click(sender, e);
        }

        private void cmbClient_SelectedIndexChanged(object sender, EventArgs e)
        {
            bool enabled = false;
            if (cmbClient.SelectedItem != null)
            {
                _fiscalClient = (FiscalClient) cmbClient.SelectedItem;
                if (_fiscalClient.StoreName != "Empty")
                    enabled = true;
            }

            btnStartDocument.Enabled = enabled;
            btnStoreDocument.Enabled = enabled & _fiscalResponse != null;
            txtFiscalResponse.Enabled = enabled;
            picQRCode.Enabled = enabled;
        }

        private void cmdReload_Click(object sender, EventArgs e)
        {
            // clear all clients
            cmbClient.Items.Clear();
            // add one empty client to switch back to empty client
            cmbClient.Items.Add(new FiscalClient() { StoreName = "Empty" });

            try
            {
                // get available clients from service
                FiscalClient[] clients = _serviceClient.GetClients();
                // and add it to the Combobox
                cmbClient.Items.AddRange(clients);
            }
            catch (Exception ex)
            {
                MessageBox.Show($"Error {ex} loading clients from service. Please use Reload Button to use again.");
            }
        }

        private void btnStartDocument_Click(object sender, EventArgs e)
        {
            // here you create a document and get a transaction number from the tse (is done by retailforce service)
            // store fiscalresponse for further usage --> you'll need some information when you want to store the document
            _fiscalResponse = _serviceClient.CreateDocument(_fiscalClient.UniqueClientId, DocumentType.Receipt);
            // IMPORTANT: In real production environment you have to create the document at the beginning of 
            //    the transaction (when first item is added to basket, or any other type of row).
            btnStoreDocument.Enabled = true;
            btnStartDocument.Enabled = false;

            txtFiscalResponse.Text = string.Empty;
            picQRCode.Image = null;
            lblPrintMessage.Text = string.Empty;
        }

        private void btnStoreDocument_Click(object sender, EventArgs e)
        {
            // first check if you have created the document already.
            // you can only store a document which was initially created at the service.
            if (_fiscalResponse == null) return;

            // ------------------ JUST FOR INFO - > NOT FOR PRODUCTION CODE -----------------------------------------------

            // just for information --> to get vat identifications of the system call
            // you'll get an array of all available vat definitions
            Vat[] vats = _serviceClient.GetSupportedVatDefinitions(_fiscalClient.UniqueClientId);

            // END -------------- JUST FOR INFO - > NOT FOR PRODUCTION CODE -----------------------------------------------

            // you can create the document by adding the fiscal response to the constructor.
            // now every information for storing the document (for the basic document transfer process) is added to the document.
            // and now you have to add the positions and other information
            Document document = new Document(_fiscalResponse);

            // at first add header information ------------------------------------------------------------------
            // create date (when document was created, in this sample just now, but should be stored when adding the first item to the basket)
            document.CreateDate = DateTime.Now;
            // book date (when document is stored -> use store date/time of your system
            document.BookDate = DateTime.Now;
            // document type -> in this case receipt -> you have to implement available types
            document.DocumentType = DocumentType.Receipt;
            // the guid for the document, if there is a guid also in your system use it please
            document.DocumentGuid = Guid.NewGuid();
            // use date because no number generation available -> here you should add your internal document identification -> must be unique
            document.DocumentId = DateTime.Now.ToString("O");
            // this is not a training document
            document.IsTraining = false;
            // the clientId for this document -> must be the same as at the call of CreateDocument
            document.UniqueClientId = _fiscalClient.UniqueClientId;
            // the user --> if you have this information --> use information of your system
            document.User = new User()
            {
                Caption = "Verkäufer1",
                Id = "1"
            };

            // now add an item position (its just a sample :))
            document.Positions.Add(new DocumentPositionItem()
            {
                // this is the unique item id for this position
                ItemId = "12345",
                // the caption of the item
                ItemCaption = "Artikel 1",
                // if available the item code (like ean13, qrcode, ean8, ean128, etc.) for the item (global tracking identification number)
                GTIN = "EAN1367890123",
                // the gross value of the position (price * quantity = value) including tax and including all discounts
                GrossValue = 11.9m,
                // the net value of the position (price * quantity = value) excluding tax and including all discounts
                NetValue = 10m,
                // the tax value of the position (price * quantity = value) including all discounts
                TaxValue = 1.9m,
                // the base gross value of the position (price * quantity = value) including tax and excluding all discounts
                BaseGrossValue = 11.9m,
                // the base net value of the position (price * quantity = value) excluding tax and excluding all discounts
                BaseNetValue = 10m,
                // the base tax value of the position (price * quantity = value) excluding all discounts
                BaseTaxValue = 1.9m,
                // the vat percentage
                VatPercent = 19m,
                // the id of the vat in the retailforce system -> to obtain available vat definitions see beginning of this procedure
                VatIdentification = 1,
                // the quantity of this row
                Quantity = 1,
                // the quantity unit 
                QuantityUnit = new QuantityUnit() { Id = "STK" },
                // true if item was used / eaten inhouse; otherwise false --> necessary for take away
                InHouse = false,
                // the business transaction type for the normal revenue, please take a look a available business transaction types
                BusinessTransactionType = BusinessTransactionType.Revenue,
            });

            // add at least one payment --> at document with type receipt payments sum = position sum
            document.Payments.Add(new DocumentPayment()
            {
                // the amount of the payment
                Amount = 11.9m,
                // the caption of the payment
                Caption = "Bar",
                // the currency of the payment (must be currency iso code 4217)
                CurrencyIsoCode = "EUR",
                // the payment type of the payment -> must be set.
                PaymentType = PaymentType.Cash,
            });

            List<ValidationError> errors = _serviceClient.ValidateDocument(document);
            foreach(ValidationError error in errors)
            {
                if (error.ErrorLevel == ErrorLevel.Error)
                {
                    // validation error occured -> will not be at this sample
                    // just for programming flow information
                    // 
                    // you have to correct the error before you call store document, otherwise the command will fail
                    return;
                }
            }

            try
            {
                FiscalResponse response = _serviceClient.StoreDocument(document);
                // set response to output window 
                txtFiscalResponse.Text = JsonConvert.SerializeObject(response, Formatting.Indented);

                // show user message if set to the user
                if (!string.IsNullOrEmpty(response.UserMessage))
                    MessageBox.Show(response.UserMessage);

                // ALSO IMPORTANT: if print message is set of response (from V1.2) -> you have to print out print message
                if (!string.IsNullOrEmpty(response.PrintMessage))
                    lblPrintMessage.Text = response.PrintMessage;
                
                // now get qr code -------------------

                // qr code can be get by extension function QrCodeDataString();
                string qrCode = response.QrCodeDataString();

                // you can get the qr code as picture by the service (or you format the qr code by yourself)
                string qrCodePicture = _serviceClient.GetQrCodeBase64(qrCode, 1);

                // now get bytes from this encoded string and write to memorystream to show the picture.
                if (!string.IsNullOrEmpty(qrCodePicture))
                {
                    Image image;
                    byte[] bytes = Convert.FromBase64String(qrCodePicture);
                    using (MemoryStream stream = new MemoryStream(bytes))
                    {
                        image = Image.FromStream(stream);
                    }
                    picQRCode.Image = image;
                }

                // -----------------------------------

                // cleanup resources
                _fiscalResponse = null;
                btnStoreDocument.Enabled = false;
                btnStartDocument.Enabled = true;
            }
            catch (Exception ex)
            {
                MessageBox.Show($"Error {ex} storing document to service. Please try again.");
            }
        }
    }
}
